@@ -20,6 +20,7 @@ import time
 import pygobject
 import sendSMS
 import locale
+import dbus
 from portrait import FremantleRotation
 
 # By using apt-cache search, the following packages are required:
@@ -28,15 +29,18 @@ from portrait import FremantleRotation
 _ = i18n.language.gettext	# Internationization support
 
 class StrokeOrder():
+	_PORTRAIT, _LANDSCAPE = ('portrait', 'landscape')
+	
 	strokesInputted = gtk.Label()	# Define label for displaying strokes
 	strokesInputted.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#002090"))	# Modify label font color
 	strokesCodeInputted = ""	# Define variable for stroke codes being inputted
 	eb = gtk.EventBox()
 	eb.add(strokesInputted)
 	eb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#AADDFF"))	# Modify label background color
+	eb.set_size_request(800,50)
 	
 	candidateChars = hildon.TouchSelector(text = True)	# Define TouchSelector control for displaying candidate characters
-	
+	smsTextLabel = gtk.Label()	
 	pannableArea = hildon.PannableArea()
 	pannableArea.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
 	
@@ -53,6 +57,8 @@ class StrokeOrder():
 	sendTo.set_placeholder(_("StrokeOrder_sendToPlaceHolder"))	# Set the info text for display when phone number text field is empty
 	textboxInFocus = "textArea"	# Set input focus to the text area initially
 
+	sendToNumbers = ""	# Variable for remembering the contact numbers when user switches back and forth the pages
+	
 	con=sqlite3.connect("/opt/StrokeOrder/IM.db")	# Connect to stroke lookup database
 	con.text_factory=str
 	cur = con.cursor()			
@@ -62,7 +68,29 @@ class StrokeOrder():
 
 	def __init__(self):	
 		self.main()
-
+			
+	def _on_orientation_signal(self, orientation, stand, face, x, y, z):
+		self.AppWidth, self.AppHeight = self.currentwindow.get_size()		
+		self.orientation = orientation
+		print "Width: " + str(self.AppWidth) + ", Height: " + str(self.AppHeight)		
+		if orientation == "portrait":
+			self.resizeControls(480,744)
+		else:
+			self.resizeControls(800,424)
+		
+	def resizeControls(self, AppWidth, AppHeight):
+		self.eb.set_size_request(AppWidth,50)
+		self.buttonHBox1.set_size_request(AppWidth,int(AppHeight*0.165))
+		self.buttonHBox2.set_size_request(AppWidth,int(AppHeight*0.165))
+		
+		self.EngButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+		self.SymButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+		self.SMSPageButton.set_size_request(int(AppWidth/3),int(AppHeight*0.165))
+		
+		self.textAreaButtonsHBox.set_size_request(AppWidth,int(AppHeight*0.165))
+		self.candidateChars.set_size_request(150,int(AppHeight*0.2830))
+		self.candidateCharsAndtextAreaHBox.set_size_request(int(AppWidth),int(AppHeight*0.2830))			
+	
 	def getCurrentLocale(self):
 		# Return the current system locale
 		lc, encoding = locale.getdefaultlocale()
@@ -102,8 +130,18 @@ class StrokeOrder():
 				self.candidateChars.append_text(row[0])	# Generate associated phrases selection list
 		
 	def main(self):	
+		system_bus = dbus.Bus.get_system()
+		system_bus.add_signal_receiver(self._on_orientation_signal, \
+			signal_name='sig_device_orientation_ind', \
+			dbus_interface='com.nokia.mce.signal', \
+			path='/com/nokia/mce/signal')
+			
 		program = hildon.Program.get_instance()
 		window = hildon.StackableWindow()
+				
+		self.currentwindow = window		
+
+		self.AppWidth, self.AppHeight = self.currentwindow.get_size()
 		# scrolledWindow = gtk.ScrolledWindow(hadjustment=None, vadjustment=None)
 		# scrolledWindow.show()
 		# scrolledWindow.set_policy('automatic','automatic')
@@ -120,13 +158,13 @@ class StrokeOrder():
 		
 		# Creating user interface
 		
-		buttonHBox1 = gtk.HBox(False, 0)
-		buttonHBox2 = gtk.HBox(False, 0)
-		EngSymButtonVBox = gtk.VBox(False, 0)
-		candidateCharsAndtextAreaHBox = gtk.HBox(False, 0)
-		strokesInputtedAndEngSymButtonHBox = gtk.HBox(False, 0)	
-		sendToAndSelectContactButtonHBox = gtk.HBox(False, 0)	
+		self.buttonHBox1 = gtk.HBox(False, 0)
+		self.buttonHBox2 = gtk.HBox(False, 0)
+		self.SymButtonVBox = gtk.VBox(False, 0)
+		self.candidateCharsAndtextAreaHBox = gtk.HBox(False, 0)		
+		
 		self.textAreaButtonsVBox = gtk.VBox(False, 0)	
+		self.textAreaButtonsHBox = gtk.HBox(False, 0)	
 		 
 		vbox = gtk.VBox(False, 0)
 		
@@ -150,13 +188,13 @@ class StrokeOrder():
 		FiveButton.set_label("乛")   
 		FiveButton.connect("clicked", self.StrokeClicked, 5)
 
-		EngButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		EngButton.set_label(_("StrokeOrder_EngMode"))   
-		EngButton.connect("clicked", self.StrokeClicked, 6)
+		self.EngButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		self.EngButton.set_label(_("StrokeOrder_EngMode"))   
+		self.EngButton.connect("clicked", self.StrokeClicked, 6)
 
-		SymButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		SymButton.set_label(_("StrokeOrder_SymMode"))   
-		SymButton.connect("clicked", self.StrokeClicked, 7)
+		self.SymButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		self.SymButton.set_label(_("StrokeOrder_SymMode"))   
+		self.SymButton.connect("clicked", self.StrokeClicked, 7)
 		
 		WildcardButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		WildcardButton.set_label("？")   
@@ -170,69 +208,63 @@ class StrokeOrder():
 		DelButton.set_label("←")   
 		DelButton.connect("clicked", self.DelClicked, window)
 
-		ClearButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		ClearButton.set_label(_("StrokeOrder_Clear"))   
-		ClearButton.connect("clicked", self.ClearClicked)
+		self.SMSPageButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		self.SMSPageButton.set_label(_("StrokeOrder_Send"))   
+		self.SMSPageButton.connect("clicked", self.SMSPageClicked)
 		
-		sendSMSButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		sendSMSButton.set_label(_("StrokeOrder_Send"))
-		sendSMSButton.connect("clicked", self.sendSMSPreCheck, window)
+		NewButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		NewButton.set_label(_("StrokeOrder_New"))   
+		NewButton.connect("clicked", self.NewClicked)
 		
-		selectContactButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
-		selectContactButton.set_label(_("StrokeOrder_SelectContact"))
-		selectContactButton.connect("clicked", self.selectContactClicked)
-		
-		self.sendTo.connect("grab-focus", self.setCurrentTextboxInFocus, "sendTo")
-		
-		buttonHBox1.pack_start(OneButton, True, True, 0)
-		buttonHBox1.pack_start(TwoButton, True, True, 0)
-		buttonHBox1.pack_start(ThreeButton, True, True, 0)
-		buttonHBox1.set_size_request(800,75)
-		buttonHBox2.pack_start(FourButton, True, True, 0)
-		buttonHBox2.pack_start(FiveButton, True, True, 0)
-		buttonHBox2.pack_start(WildcardButton, True, True, 0)
-		
-		buttonHBox2.set_size_request(800,75)
+		self.buttonHBox1.pack_start(OneButton, True, True, 0)
+		self.buttonHBox1.pack_start(TwoButton, True, True, 0)
+		self.buttonHBox1.pack_start(ThreeButton, True, True, 0)
 
-		strokesInputtedAndEngSymButtonHBox.pack_start(EngButton, False, True, 0)		
-		strokesInputtedAndEngSymButtonHBox.pack_start(self.eb, True, True, 0)
-		strokesInputtedAndEngSymButtonHBox.pack_start(SymButton, False, True, 0)		
+		self.buttonHBox2.pack_start(FourButton, True, True, 0)
+		self.buttonHBox2.pack_start(FiveButton, True, True, 0)
+		self.buttonHBox2.pack_start(WildcardButton, True, True, 0)				
 		
+		print "Width: " + str(self.AppWidth) + ", Height: " + str(self.AppHeight)
+	
 		self.textArea.connect("grab-focus", self.setCurrentTextboxInFocus, "textArea")
 		self.textAreaBuffer.connect("changed", self.refreshCharLeft)
-		
+
+		self.textAreaButtonsHBox.pack_start(self.EngButton, True, True, 0)	
+		self.textAreaButtonsHBox.pack_start(self.SymButton, True, True, 0)	
+		# self.textAreaButtonsHBox.pack_start(CopyButton, True, True, 0)	
+		self.textAreaButtonsHBox.pack_start(self.SMSPageButton, True, True, 0)						
+						
 		self.pannableArea.add_with_viewport(self.textArea)
-		self.candidateChars.set_size_request(150,200) 
-		candidateCharsAndtextAreaHBox.pack_start(self.candidateChars, False, True, 0)
-		candidateCharsAndtextAreaHBox.pack_start(self.pannableArea, True, True, 0)
+		self.candidateCharsAndtextAreaHBox.pack_start(self.pannableArea, True, True, 0)
+		self.candidateCharsAndtextAreaHBox.pack_start(self.candidateChars, False, True, 0)		
 
 		self.textAreaButtonsVBox.pack_start(self.charLeft, False, False, 0)
 		self.textAreaButtonsVBox.pack_start(DelButton, True, True, 0)	
-		self.textAreaButtonsVBox.pack_start(ClearButton, True, True, 0)	
-		self.textAreaButtonsVBox.pack_start(CopyButton, True, True, 0)	
-		self.textAreaButtonsVBox.set_size_request(120,200)
-		candidateCharsAndtextAreaHBox.pack_start(self.textAreaButtonsVBox, False, False, 0)	
-		
-		sendToAndSelectContactButtonHBox.pack_start(self.sendTo, True, True, 0)		
-		sendToAndSelectContactButtonHBox.pack_start(sendSMSButton, False, False, 0)		
-		sendToAndSelectContactButtonHBox.pack_start(selectContactButton, False, False ,0)
-		
+		self.textAreaButtonsVBox.pack_start(CopyButton, True, True, 0)
+
+		self.candidateCharsAndtextAreaHBox.pack_start(self.textAreaButtonsVBox, False, False, 0)	
+
 		self.candidateChars.connect("changed", self.CharSelected)
+		self.strokesCodeInputted = "6"
+		self.queryString(self.strokesCodeInputted)
+		self.strokesInputted.set_text( self.ConvertCodeToChar(self.strokesCodeInputted) )	# Display the inputted strokes
 		
-		vbox.pack_start(buttonHBox1, False, True ,0)
-		vbox.pack_start(buttonHBox2, False, True ,0)
-		vbox.pack_start(strokesInputtedAndEngSymButtonHBox, False, False ,0)
-		vbox.pack_start(sendToAndSelectContactButtonHBox, False, False, 0)		
-		vbox.pack_start(candidateCharsAndtextAreaHBox, True, True, 0)
+		vbox.pack_start(self.eb, False, False ,0)				
+		vbox.pack_start(self.candidateCharsAndtextAreaHBox, True, True, 0)		
+		vbox.pack_start(self.buttonHBox1, False, True ,0)
+		vbox.pack_start(self.buttonHBox2, False, True ,0)		
+		vbox.pack_start(self.textAreaButtonsHBox, False, True, 0)		
+		
+		self.resizeControls(800,424)
 		
 		window.set_title(_("StrokeOrder_AppName")) 
 		window.connect("delete_event",self.app_preclose)
-		menu = self.create_menu(window) 
+		menu = self.create_menu(window, "MainPage") 
 		window.set_app_menu(menu)	
 		window.add(vbox)	
-		window.show_all() 	
-		self.textArea.grab_focus()
-		gtk.main()	
+		window.show_all()		
+		self.textArea.grab_focus()		
+		gtk.main()			
 		
 	def aboutProduct(self, AboutProductButton, window):
 		# Show the About this Product dialog box
@@ -242,16 +274,190 @@ class StrokeOrder():
 		response = dialog.run()
 		dialog.destroy()
 
-	def create_menu(self, window):
+	def SMSPageClicked(self, window):
+		smsText = self.textAreaBuffer.get_text(self.textAreaBuffer.get_start_iter(), self.textAreaBuffer.get_end_iter(), False)
+		
+		if (smsText!=""):
+			sendSMSPage = hildon.StackableWindow()
+			sendSMSPage.set_title(_("StrokeOrder_Send"))
+			
+			vbox = gtk.VBox(False, 0)		
+			
+			smsText = unicode(smsText,"utf-8")
+			msglength = len(smsText)
+			textToBeSent = ""
+			for i in range(1, msglength/70+2 ):
+				textToBeSent = textToBeSent + "＊＊＊ " + _("StrokeOrder_MessageCount") + str(i) + " ＊＊＊\n" + smsText[(i*70-70):(i*70)] + "\n\n"
+			
+			self.smsTextLabel.set_text(_("StrokeOrder_MessagePreview") + "\n\n" + textToBeSent)
+			self.smsTextLabel.set_alignment(0,0)						
+			# self.smsTextLabel.set_line_wrap(True)
+		
+			self.smsTextLabel.modify_fg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#000000"))	# Modify label font color	
+			evtb = gtk.EventBox()
+			evtb.add(self.smsTextLabel)
+			evtb.modify_bg(gtk.STATE_NORMAL, gtk.gdk.color_parse("#999999"))	# Modify label background color		
+
+			pannableAreaForSMSText = hildon.PannableArea()
+			pannableAreaForSMSText.set_property("mov-mode", hildon.MOVEMENT_MODE_BOTH)
+			pannableAreaForSMSText.add_with_viewport(evtb)
+			pannableAreaForSMSText.set_size_request(800,100)
+			
+			sendToAndSelectContactButtonHBox = gtk.HBox(False, 0)
+						
+			self.sendToBuffer.set_text(self.sendToNumbers)			# Retrieve remembered contact numbers, if user switches pages back and forth
+			self.sendToBuffer.connect("changed",self.rememberContactNumbers)	# In case sendTo text box content has been changed, call the rememberContactNumbers function
+			self.sendTo.set_buffer(self.sendToBuffer)			
+			self.sendTo.connect("grab-focus", self.setCurrentTextboxInFocus, "sendTo")
+
+			sendSMSButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			sendSMSButton.set_label(_("StrokeOrder_Send"))
+			sendSMSButton.connect("clicked", self.sendSMSPreCheck, self.currentwindow)		
+		
+			selectContactButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			selectContactButton.set_label(_("StrokeOrder_SelectContact"))
+			selectContactButton.connect("clicked", self.selectContactClicked)
+			
+			OneButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			OneButton.set_label("１")
+			OneButton.connect("clicked", self.numpadButtonClicked, 1)
+			
+			TwoButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			TwoButton.set_label("２")
+			TwoButton.connect("clicked", self.numpadButtonClicked, 2)
+			
+			ThreeButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			ThreeButton.set_label("３")
+			ThreeButton.connect("clicked", self.numpadButtonClicked, 3)		
+			
+			FourButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			FourButton.set_label("４")
+			FourButton.connect("clicked", self.numpadButtonClicked, 4)		
+			
+			FiveButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			FiveButton.set_label("５")
+			FiveButton.connect("clicked", self.numpadButtonClicked, 5)
+
+			SixButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			SixButton.set_label("６")
+			SixButton.connect("clicked", self.numpadButtonClicked, 6)
+
+			SevenButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			SevenButton.set_label("７")
+			SevenButton.connect("clicked", self.numpadButtonClicked, 7)
+			
+			EightButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			EightButton.set_label("８")
+			EightButton.connect("clicked", self.numpadButtonClicked, 8)
+
+			NineButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			NineButton.set_label("９")
+			NineButton.connect("clicked", self.numpadButtonClicked, 9)
+			
+			PlusButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			PlusButton.set_label("＋")
+			PlusButton.connect("clicked", self.numpadButtonClicked, "+")
+			
+			ZeroButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			ZeroButton.set_label("０")
+			ZeroButton.connect("clicked", self.numpadButtonClicked, "0")
+			
+			SemicolonButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			SemicolonButton.set_label("；")
+			SemicolonButton.connect("clicked", self.numpadButtonClicked, ";")
+
+			DelButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+			DelButton.set_label("←")   
+			DelButton.connect("clicked", self.DelClicked, window)
+			
+			sendToAndSelectContactButtonHBox.pack_start(self.sendTo, True, True, 0)						
+			sendToAndSelectContactButtonHBox.pack_start(selectContactButton, False, False ,0)
+			
+			numpadHBox1 = gtk.HBox(False, 0)
+			numpadHBox1.pack_start(OneButton,True,True,0)
+			numpadHBox1.pack_start(TwoButton,True,True,0)
+			numpadHBox1.pack_start(ThreeButton,True,True,0)
+					
+			numpadHBox2 = gtk.HBox(False, 0)
+			numpadHBox2.pack_start(FourButton,True,True,0)
+			numpadHBox2.pack_start(FiveButton,True,True,0)
+			numpadHBox2.pack_start(SixButton,True,True,0)
+			
+			numpadHBox3 = gtk.HBox(False, 0)
+			numpadHBox3.pack_start(SevenButton,True,True,0)
+			numpadHBox3.pack_start(EightButton,True,True,0)
+			numpadHBox3.pack_start(NineButton,True,True,0)
+			
+			numpadHBox4 = gtk.HBox(False, 0)
+			numpadHBox4.pack_start(PlusButton,True,True,0)
+			numpadHBox4.pack_start(ZeroButton,True,True,0)
+			numpadHBox4.pack_start(DelButton,True,True,0)		
+			
+			numpadHBox5 = gtk.HBox(False, 0)
+			numpadHBox5.pack_start(SemicolonButton,True,True,0)
+			numpadHBox5.pack_start(sendSMSButton,True,True,0)
+			
+			numpad = gtk.VBox(False, 0)
+			numpad.pack_start(numpadHBox1,True,True,0)
+			numpad.pack_start(numpadHBox2,True,True,0)
+			numpad.pack_start(numpadHBox3,True,True,0)
+			numpad.pack_start(numpadHBox4,True,True,0)
+			numpad.pack_start(numpadHBox5,True,True,0)
+			
+			textAndnumpadVBox = gtk.VBox(False, 0)
+			textAndnumpadVBox.pack_start(pannableAreaForSMSText, True, True, 0)
+			textAndnumpadVBox.pack_start(numpad, True, True, 0)				
+			
+			vbox.pack_start(sendToAndSelectContactButtonHBox, False, False, 0)						
+			vbox.pack_start(textAndnumpadVBox, True, True, 0)		
+			
+			sendSMSPage.add(vbox)
+			menu = self.create_menu(sendSMSPage, "SMSPage") 
+			sendSMSPage.set_app_menu(menu)				
+			sendSMSPage.show_all()
+		else:					
+			banner = hildon.hildon_banner_show_information(window,"" , _("StrokeOrder_Error_BlankMessage"))
+			
+	def rememberContactNumbers(self, widget):
+		# For remembering the contact numbers when user switches back and forth the pages
+		self.sendToNumbers = self.sendToBuffer.get_text(self.sendToBuffer.get_start_iter(), self.sendToBuffer.get_end_iter())	# For remembering the contact numbers when user switches back and forth the pages
+	
+	def numpadButtonClicked(self, window, number):
+		buffer = self.sendTo.get_buffer() 
+		buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + str(number) + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
+		self.sendTo.set_buffer(buffer)			
+			
+	def create_menu(self, window, page):
 		# Create the application menu
 		
 		menu = hildon.AppMenu()   
 		
+		CopyButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		CopyButton.set_label(_("StrokeOrder_Copy"))   
+		CopyButton.connect("clicked", self.CopyClicked, window)
+		
+		PasteButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		PasteButton.set_label(_("StrokeOrder_Paste"))   
+		PasteButton.connect("clicked", self.pasteClicked, window)    
+		
 		AboutButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
 		AboutButton.set_label(_("StrokeOrder_AboutProduct"))   
 		AboutButton.connect("clicked", self.aboutProduct, window)    
+		
+		NewButton = hildon.GtkButton(gtk.HILDON_SIZE_AUTO)
+		NewButton.set_label(_("StrokeOrder_New"))   
+		NewButton.connect("clicked", self.NewClicked)			
+		if page == "MainPage":
+			NewButton.set_sensitive(True)
+		elif page == "SMSPage":
+			NewButton.set_sensitive(False)	# Disable button if user is using SMS page
+		menu.append(NewButton)
+			
+		menu.append(AboutButton)   		
+		
+		menu.append(CopyButton)
+		menu.append(PasteButton)		
 
-		menu.append(AboutButton)   
 		menu.show_all()
 		return menu
 
@@ -290,30 +496,58 @@ class StrokeOrder():
 		
 		self.queryString(self.strokesCodeInputted)
 		
+	def pasteClicked(self, PasteButton, window):
+		# Triggered when user clicks the Paste button from the menu
+		clip = gtk.Clipboard()
+		data = clip.wait_for_contents('UTF8_STRING')
+		text = str(data.get_text())
+		if self.textboxInFocus == "textArea":
+			buffer = self.textArea.get_buffer()
+		elif self.textboxInFocus == "sendTo":
+			buffer = self.sendTo.get_buffer() 
+			
+		selectedText = buffer.get_selection_bounds()
+		if selectedText!=(): # If there is selected text, delete the selected text
+			buffer.delete_selection(True,True)
+		buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
+ 
+		if self.textboxInFocus == "textArea":
+			self.textArea.set_buffer(buffer)
+		elif self.textboxInFocus == "sendTo":
+			self.sendTo.set_buffer(buffer)
+	
 	def CharSelected(self, Selector, column):
 		# Triggered once a candidate / English / Numeric / Symbol character is selected from the TouchSelector control.
 		# Send the selected text to the text area.
 		
 		text = Selector.get_current_text()
-		if self.textboxInFocus == "textArea":	# If text area is having input focus, insert text to the current cursor position
-			buffer = self.textArea.get_buffer()
-			buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))			
-			self.textArea.set_buffer(buffer)	
-		elif self.textboxInFocus == "sendTo":	# If phone number field is having input focus, insert text to the current cursor position
-			buffer = self.sendTo.get_buffer() 
-			buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
-			self.sendTo.set_buffer(buffer)							
 		
-		self.strokesInputted.set_text("")	# Clear the stroke display area once the text is inserted
+#		if self.textboxInFocus == "textArea":	# If text area is having input focus, insert text to the current cursor position
+		buffer = self.textArea.get_buffer()
+		buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))			
+		self.textArea.set_buffer(buffer)	
+#		elif self.textboxInFocus == "sendTo":	# If phone number field is having input focus, insert text to the current cursor position
+#			buffer = self.sendTo.get_buffer() 
+#			buffer.set_text(buffer.get_text(buffer.get_start_iter(), buffer.get_iter_at_mark(buffer.get_insert()), False) + text + buffer.get_text(buffer.get_iter_at_mark(buffer.get_insert()), buffer.get_end_iter(), False))
+#			self.sendTo.set_buffer(buffer)							
+				
 		if not "6" in self.strokesCodeInputted and not "7" in self.strokesCodeInputted:	# If any of the stroke 1-5 is pressed, once text is inserted into the text area, clear the stroke sequence variable and associated phrases
+			text = unicode(text, "utf-8")		
+			if len(text)>1:				
+				text = text[-1]
+			self.strokesInputted.set_text(_("StrokeOrder_SelectAssoc1") + text + _("StrokeOrder_SelectAssoc2"))	# Clear the stroke display area once the text is inserted
 			self.strokesCodeInputted = ""
 			self.queryString("")			
 			self.queryAssoc(text)
-
+	
 	def CopyClicked(self, CopyButton, window):
 		# Copy the text area content to the clipboard
 		clipboard = gtk.Clipboard()
-		buffer = self.textArea.get_buffer()
+		if self.textboxInFocus == "textArea":
+			buffer = self.textArea.get_buffer()
+		elif self.textboxInFocus == "sendTo":
+			buffer = self.sendTo.get_buffer()
+			
 		selectedText = buffer.get_selection_bounds()
 		if selectedText==():	# If some text is selected, copy the selected text to clipboard only
 			textToBeCopied = buffer.get_text(buffer.get_start_iter(), buffer.get_end_iter(), False)
@@ -327,7 +561,7 @@ class StrokeOrder():
 
 	def DelClicked(self, LeftButton, window):		
 		if self.textboxInFocus == "sendTo":		# Set buffer according to current input focus
-			buffer = self.sendTo.get_buffer()
+			buffer = self.sendTo.get_buffer()			
 		elif self.textboxInFocus == "textArea":
 			buffer = self.textArea.get_buffer()
 			
@@ -350,20 +584,21 @@ class StrokeOrder():
 		msgCount = len(msg) / 70 + 1
 		self.charLeft.set_text( str( charactersLeft ) + "(" + str(msgCount) + ")" )
 							
-	def ClearClicked(self, widget): 
+	def NewClicked(self, widget): 
 		# Reset everything back to the very beginning
 		text = ""
 		buffer = self.textArea.get_buffer()
 		buffer.set_text(text)
 		self.textArea.set_buffer(buffer)
 		
-		self.strokesCodeInputted = ""
-		self.strokesInputted.set_text("")
-		
 		buffer = self.sendTo.get_buffer()
 		buffer.set_text("")
 		
-		self.queryString("")
+		self.smsTextLabel.set_text("")
+		
+		self.strokesCodeInputted = "6"
+		self.queryString(self.strokesCodeInputted)
+		self.strokesInputted.set_text( self.ConvertCodeToChar(self.strokesCodeInputted) )	# Display the inputted strokes
 				
 	def selectContactClicked(self, widget):
 		# Pops up the Address Book, select a contact and phone number will be inserted to the phone number text field
@@ -409,25 +644,35 @@ class StrokeOrder():
 			finalval = osso_abook.osso_abook_contact_field_get_display_value(c_field)
 	#            print oldbuffer
 			if oldbuffer == "":
-				self.sendToBuffer.set_text(finalval)
+				if finalval != None:
+					self.sendToBuffer.set_text(str(finalval))
 			else:
-				newbuffer = oldbuffer +";" + finalval
-				self.sendToBuffer.set_text(newbuffer)
-
+				if finalval != None:
+					newbuffer = oldbuffer +";" + str(finalval)
+					self.sendToBuffer.set_text(newbuffer)
+		
 	def sendSMSPreCheck(self, widget, window):
 		# Before sending out SMS, detect long message and break it down into multiple ones.
 		start, end = self.textAreaBuffer.get_bounds()
 		text = unicode(self.textAreaBuffer.get_text(start,end), "utf-8")
-		msglength = len(text)
-		for i in range(1, msglength/70+2 ):
-			textToBeSent = text[(i*70-70):(i*70)]
-			self.sendSMS(widget, window, textToBeSent )
-			print "Loop: " + str(i)
-			print "Message length: " + str(msglength)
-			print "Total no. of messages to be sent: " + str(msglength/70+1)
-			print "Text to be sent:\n" + textToBeSent
-		os.system("sudo pkill -f rtcom-messaging-ui")
-			
+		start, end = self.sendToBuffer.get_bounds()
+		number = self.sendToBuffer.get_text(start,end)
+		
+		if text=="":			  
+			banner = hildon.hildon_banner_show_information(window,"" , _("StrokeOrder_Error_BlankMessage"))
+		elif number == "":	# If phone number is not specified...
+			banner = hildon.hildon_banner_show_information(window,"" , _("StrokeOrder_Error_NoContact"))
+			print "error, no number specified!"
+		else:
+			msglength = len(text)
+			for i in range(1, msglength/70+2 ):
+				textToBeSent = text[(i*70-70):(i*70)]
+				self.sendSMS(widget, window, textToBeSent )
+				print "Loop: " + str(i)
+				print "Message length: " + str(msglength)
+				print "Total no. of messages to be sent: " + str(msglength/70+1)
+				print "Text to be sent:\n" + textToBeSent
+			os.system("sudo pkill -f rtcom-messaging-ui")
 
 	def sendSMS(self, widget, window, text):
 		# Send SMS to the specified recipient(s)
@@ -475,5 +720,11 @@ class StrokeOrder():
 		# Close database connection before closing the application.
 		self.con.close()
 		gtk.main_quit(None)
-if __name__ == "__main__":
+		
+if __name__ == "__main__":	
+#	checkInstanceCMD = "ps | grep \"python /opt/StrokeOrder/StrokeOrder.py\" | wc -l"
+#	fin,fout = os.popen4(checkInstanceCMD)
+#	checkInstanceResult = str(fout.read().replace("\n",""))	
+#	if checkInstanceResult=="2":	# If only two matches found, i.e. one for the grep command and another one the program itself, continue program execution
+#		StrokeOrder()
 	StrokeOrder()
\ No newline at end of file
